@@ -1,7 +1,8 @@
 #!/usr/bin/env python3
 
+from pprint import pprint  # noqa: W0611 F401
+
 from .component import Component
-from .message import Message
 
 
 class Filter(Component):
@@ -29,25 +30,6 @@ class ChannelChain(Filter):
                 return test_name, message
 
 
-class LifeCycleSignal(Filter):
-
-    async def filter(self, messages):
-        async for channel, message in messages:
-            yield (channel, message)
-
-            if not self.running:
-                break
-
-        settings = self.settings()
-        if settings.event == 'finished':
-            yield (settings.get('channel', 'finished'), LifeCycleMessage('finished'))
-
-
-class LifeCycleMessage(Message):
-    def __init__(self, event, **kwargs):
-        super().__init__(event=event, **kwargs)
-
-
 class MessageMangleFilter(Filter):
 
     def deepget(self, mapping, key):
@@ -74,3 +56,59 @@ class MessageMangleFilter(Filter):
             key = list(spec.keys())[0]
             self.deepset(message, key, spec[key].format(**message))
         return channel, message
+
+
+class MessageStreamFilter(Filter):
+
+    def configure(self, **settings):
+        # import pudb
+        # pudb.set_trace()
+        settings.setdefault('name', self.id.split('.')[-1])
+        for prop in ['add', 'wait']:
+            if prop in settings:
+                value = settings[prop]
+                if isinstance(value, str):
+                    value = [value]
+                settings[prop] = set(value)
+                break
+
+        self.settings = settings
+        self.stream = MessageStream(self.name)
+        self.seen = set()
+        return settings
+
+    async def process(self, channel, message):
+        settings = self.settings()
+        if message.meta.type in settings.get('add', []):
+            self.stream.add(message)
+        elif settings.wait:
+            if isinstance(message.meta.get('stream'), MessageStream) \
+                    and message.meta.stream.name == self.name:
+                self.seen.add(message)  # TODO: duplicate detection ?
+
+        return channel, message
+
+    def finalize(self):
+        settings = self.settings()
+        if settings.add:
+            self.stream.complete = True
+        elif settings.wait:
+            if self.stream.complete and len(self.seen) == len(self.stream):
+                if settings.action == 'message':
+                    return settings.channel, Message()
+
+
+class MessageStream:
+    def __init__(self, name):
+        self.name = name
+        self.messages = dict()
+        self.complete = False
+
+    def add(self, message):
+        message.meta.setdefault('streams', dict())
+        message.meta['streams'][self.name] = self
+        self.messages[message.meta.id] = message
+
+    def remove(self, message):
+        del message.meta['streams'][self.name]
+        del self.messages[message.meta.id]
