@@ -1,13 +1,13 @@
 #!/usr/bin/env python3
 
 import logging
-from datetime import datetime
-from uuid import uuid4
+from datetime import datetime  # maybe use udatetime
+from uuid import uuid4  # maybe use fastuuid (incompatible with PyPy)
 
 from box import Box
 
 _l = logging.getLogger(__name__)
-stop = False
+# stop = False
 
 
 class Message:
@@ -45,7 +45,7 @@ class Message:
         self.__data[name] = value
 
     def __str__(self):
-        return f'<{self.__class__.__name__} {self.meta.id}>'
+        return f'<{self.__class__.__name__} {self.get("file") or self.get("path")} {self.meta.id}>'
 
     def checkin(self, component):
         self._tag(component, 'in')
@@ -65,31 +65,26 @@ class Message:
         self._tag(component, 'dropped')
         del self._instances[self.meta.id]
         self.meta.dropped = True
+        if 'streams' in self.meta:
+            for stream in self.meta.streams.copy().values():
+                stream.remove(self)
         component.debug(f'Dropped message: {self}')
 
     def _tag(self, component, tag):
         self.meta.history.append((datetime.utcnow(), component.id, tag))
 
-    class __EOT__:
-        """Used to signal the end of transmission of a queue."""
-
-        def __repr__(self):
-            return str(self)
-
-        def __str__(self):
-            return '__EOT__'
-
-        def __bytes__(self):
-            return b'__EOT__'
-
-        def __eq__(self, other):
-            if isinstance(other, bytes):
-                return self.__bytes__() == other
-
-            elif isinstance(other, str):
-                return self.__str__() == other
-
-            else:
-                return self is other
-
-    EOT = __EOT__()
+    # def deepget(self, mapping, key):
+    #     if '.' not in key:
+    #         return mapping[key]
+    #
+    #     else:
+    #         current, remainder = key.split('.', 1)
+    #         return self.deepget(mapping[current], remainder)
+    #
+    # def deepset(self, mapping, key, value):
+    #     if '.' not in key:
+    #         mapping[key] = value
+    #
+    #     else:
+    #         current, remainder = key.split('.', 1)
+    #         self.deepset(mapping[current], remainder, value)
