@@ -50,6 +50,14 @@ class Node(Component):
 
     _dependent_statuses = set()
 
+    async def test(self, messages):
+        self.debug(f'~~~~~~ begin {messages}')
+        async for c, m in messages:
+            self.debug(f'~~~~~~ message from {messages} {c} {m}')
+            yield c, m
+            self.debug(f'~~~~~~ yielded from {messages} {c} {m}')
+        self.debug(f'~~~~~~ end {messages}')
+
     def start(self, *args):
         coroutines = [super().start(*args)]
         self.layers = ([self.inbox] +
@@ -79,9 +87,9 @@ class Node(Component):
         if self.running:
             try:
                 await self._run()
-            except Exception:
+            except Exception as exc:
                 self.exception('Fatal error')
-                self.abort()
+                self.abort(exc)
                 if False and True:  # TODO: make this configurable
                     raise
 
@@ -123,6 +131,7 @@ class Node(Component):
         """Generate an event on the first message reaching the processor."""
         first_message = True
         async for channel, message in messages:
+            # self.debug(f'*** pre-proc {channel}: {message}')
             if not self.running:
                 break
 
@@ -130,14 +139,19 @@ class Node(Component):
                 self.status = 'processing-started'
                 first_message = False
             yield (channel, message)
+            # self.debug('*** pre-proc loop')
+        # self.debug('*** pre-proc end')
 
     async def _post_processor(self, messages):
         """Generate an event on the last message leaving the processor."""
         async for channel, message in messages:
+            # self.debug(f'*** post-proc {channel}: {message}')
             if not self.running:
                 break
 
             yield (channel, message)
+            # self.debug('*** post-proc loop')
+        # self.debug('*** post-proc end')
         self.status = 'processing-finished'
 
     def get_processor(self):
@@ -151,7 +165,10 @@ class Node(Component):
         if self._retryable is None:
             self._retryable = RetryableMessages(self, messages)
         async for channel, message in self.processor(self._retryable, id_):
+            # self.debug(f'*** _retryable {channel}: {message}')
             yield channel, message
+            # self.debug('*** _retryable loop')
+        # self.debug('*** _retryable end')
 
     async def processor(self, messages, id_=None):
         """Process and yield new (channel, message)."""
@@ -432,13 +449,15 @@ class RetryableMessages:
         """Feed the inbox to the queue."""
         try:
             async for channel, self.newmsg in self.messages:
-                # self.node.debug(f'*** collector retry: feeding {self.newmsg.type} msg {self.newmsg.meta.id}')  # noqa: E501
+                # self.node.debug(f'*** collector retry: feeding {self.newmsg}')  # noqa: E501
                 self.forwarded.clear()
                 await self.queue.put((channel, self.newmsg))
                 await self.node.try_while_running(partial(self.forwarded.wait))
-            # self.node.debug('*** collector retry: feeder done')
+            # self.node.debug(f'*** collector retry: feeder done (pending: {self.pending})')
             self.exhausted = True
             if self.pending > 0:
+                self.node.debug(
+                        f'RetryableMessages waiting to exit: {self.pending} pending messages')
                 await self.node.try_while_running(partial(self.finished.wait))
             # self.node.debug('*** collector retry: feeder ready to exit')
         except ComponentInterrupted:
@@ -488,7 +507,7 @@ class RetryableMessages:
                 self.forwarded.set()
             self.pending += 1
             yield channel, message
-            # self.node.debug(f'*** collector retry: forwarded {message}')
+            # self.node.debug(f'*** collector retry: forwarded (pending: {self.pending}) {message}')  # noqa: E501
 
         # self.node.debug('*** collector retry: forwarder exiting')
         self.queue.task_done()
@@ -497,14 +516,16 @@ class RetryableMessages:
 
     def processing_done(self):
         """Signal that a message is not in the processor anymore."""
+        # self.node.debug(f'*** collector retry: processing done before (pending: {self.pending})')
         self.pending -= 1
-        # self.node.debug(f'*** collector retry: processing done {self.pending}')
+        # self.node.debug(f'*** collector retry: processing done (pending: {self.pending})')
         if self.exhausted:
             self.finished.set()
 
     async def retry(self, channel, message, wait=0.001, id_=None):
         """Insert a message back into the queue for retrying it."""
         # self.node.debug(f'*** collector retry{"" if id_ is None else " %s" % id_}: retrying {message.type} msg {message.meta.id}')  # noqa: E501
+        # self.node.debug(f'*** collector retry: retrying (pending: {self.pending}) {message}')
         await asyncio.sleep(wait)
         await self.node.loop.create_task(self.queue.put((channel, message)))
         self.processing_done()
@@ -534,6 +555,7 @@ class Outbox(pipe.Manifold):
     async def sender(self, messages):
         async for channel, message in messages:
             channel = channel or 'default'
+            # self.debug(f'*** got msg   {channel}: {message} [{self.channels[channel].id}]')  # noqa: E501
             if not self.running:
                 break
 
